@@ -3,6 +3,8 @@ use strict;
 use warnings;
 use Carp;
 use List::Util;
+use Data::Dumper qw(Dumper);
+use Data::Dumper qw(Dumper);
 use Exporter qw(import);
 our @EXPORT_OK = qw(
     incr        reduces  flatten
@@ -14,9 +16,10 @@ our @EXPORT_OK = qw(
     len         to_keys  to_vals     is_array
     is_hash     every    noop        identity
     is_empty    is_sub   flow        eql
+    to_pairs    for_each apply
 );
 
-our $VERSION = '0.13';
+our $VERSION = '0.15';
 
 use constant ARG_PLACE_HOLDER => {};
 
@@ -32,6 +35,13 @@ sub identity {
     return $args;
 }
 
+sub apply {
+    my $fn   = shift // sub {};
+    my $args = shift // [];
+
+    return $fn->(@{$args});
+}
+
 sub flow {
     my $args = [@_];
 
@@ -68,6 +78,29 @@ sub is_hash {
     return bool(ref $coll eq 'HASH');
 }
 
+sub to_pairs {
+    my $coll = shift // [];
+
+    if (is_array($coll)) {
+        return maps(sub {
+            my ($val, $idx) = @_;
+            return [$idx, $val]
+        }, $coll)
+    }
+
+    if (is_hash($coll)) {
+        return maps(sub {
+            my $key = shift;
+            return [$key, $coll->{$key}]
+        }, to_keys($coll))
+    }
+
+    return maps(sub {
+        my ($char, $idx) = @_;
+        return [$idx, $char];
+    }, to_vals($coll));
+}
+
 sub to_vals {
     my $coll = shift // [];
 
@@ -117,6 +150,16 @@ sub len {
     return length($coll);
 }
 
+sub for_each {
+    my ($fn, $coll) = @_;
+    my $idx = 0;
+
+    foreach my $val (@{ $coll }) {
+        $idx++;
+        $fn->($val, $idx - 1, $coll);
+    }
+}
+
 sub is_empty {
     my $coll = shift;
     return bool(len($coll) == 0);
@@ -467,6 +510,53 @@ Decrements the supplied number by 1
 
 =cut
 
+=head2 apply
+Calls the supplied function with the array of arguments, spreading the
+arguments into the function it invokes
+
+    my $sum_all_nums = sub {
+        my $num        = shift;
+        my $second_num = shift;
+
+        return $num + $second_num;
+    };
+
+    apply($sum_all_nums, [100, 200]);
+    # same as $sum_all_nums->(100, 200)
+
+    # => 300
+
+=cut
+
+=head2 for_each
+
+Iterates over elements of collection and invokes iteratee for each element. The iteratee is invoked with three arguments: (value, index|key, collection).
+
+
+    for_each(sub {
+       my $num = shift;
+       print $num;
+    }, [1,2,3]);
+
+
+    for_each(sub {
+       my ($num, $idx, $coll) = @_;
+       print $idx;
+    }, [1,2,3])
+
+    # 0 1 2
+
+    for_each(sub {
+       my ($num, $idx, $coll) = @_;
+       print Dumper $coll;
+    }, [1,2,3])
+
+    #   [1,2,3],
+    #   [1,2,3],
+    #   [1,2,3]
+
+=cut
+
 =head2 maps
 
 Creates an array of values by running each element in collection thru iteratee.
@@ -883,6 +973,43 @@ Creates an array of the values in a hash, of an array, or string.
 
 =cut
 
+=head2 to_pairs
+Creates an array of key-value, or idx-value pairs from arrays, hashes, and strings.
+If used on a hash, key-pair order can not be guaranteed;
+
+    to_pairs("I am a string");
+
+    # [
+    #  [0, "I"],
+    #  [1, "am"],
+    #  [2, "a"],
+    #  [3, "string"]
+    # ]
+
+    to_pairs([100, 101, 102]);
+
+    # [
+    #  [0, 100],
+    #  [1, 102],
+    #  [2, 103],
+    # ]
+
+    to_pairs({ key1 => 'value1', key2 => 'value2' });
+
+    # [
+    #   [key1, 'value1'],
+    #   [key2, 'value2']
+    # ]
+
+    to_pairs({ key1 => 'value1', key2 => { nested => 'nestedValue' }});
+
+    # [
+    #   [key1, 'value1'],
+    #   [key2, { nested => 'nestedValue' }]
+    # ]
+
+=cut
+
 =head2 uniq
 
 Creates a duplicate-free version of an array,
